@@ -17,9 +17,12 @@ import (
 	"fmt"
 	"io"
 	"math"
+	"math/bits"
 	"math/rand"
+	"reflect"
 	"strconv"
 	"testing"
+	"time"
 )
 
 func TestStringer(t *testing.T) {
@@ -129,7 +132,7 @@ func TestBitSetIsClear(t *testing.T) {
 	}
 }
 
-func TestExendOnBoundary(t *testing.T) {
+func TestExtendOnBoundary(t *testing.T) {
 	v := New(32)
 	defer func() {
 		if r := recover(); r != nil {
@@ -150,7 +153,6 @@ func TestExceedCap(t *testing.T) {
 	bmp.ClearAll()
 	d := Cap()
 	bmp.Set(d)
-
 }
 
 func TestExpand(t *testing.T) {
@@ -253,7 +255,6 @@ func TestNextClear(t *testing.T) {
 	next, found = v.NextClear(0)
 	if found || next != 0 {
 		t.Errorf("Found next clear bit as %d, it should have return (0, false)", next)
-
 	}
 }
 
@@ -300,7 +301,308 @@ func TestIterate(t *testing.T) {
 	if data[4] != 2000 {
 		t.Errorf("bug 4")
 	}
+}
 
+func TestNextSet(t *testing.T) {
+	testCases := []struct {
+		name string
+		//
+		set []uint
+		del []uint
+		//
+		startIdx uint
+		wantIdx  uint
+		wantOk   bool
+	}{
+		{
+			name:     "null",
+			set:      []uint{},
+			startIdx: 0,
+			wantIdx:  0,
+			wantOk:   false,
+		},
+		{
+			name:     "zero",
+			set:      []uint{0},
+			startIdx: 0,
+			wantIdx:  0,
+			wantOk:   true,
+		},
+		{
+			name:     "1,5",
+			set:      []uint{1, 5},
+			startIdx: 0,
+			wantIdx:  1,
+			wantOk:   true,
+		},
+		{
+			name:     "many",
+			set:      []uint{1, 65, 130, 190, 250, 300, 380, 420, 480, 511},
+			startIdx: 100,
+			wantIdx:  130,
+			wantOk:   true,
+		},
+		{
+			name:     "many-2",
+			set:      []uint{1, 65, 130, 190, 250, 300, 380, 420, 480, 511},
+			del:      []uint{130, 190, 300, 420},
+			startIdx: 100,
+			wantIdx:  250,
+			wantOk:   true,
+		},
+		{
+			name:     "last",
+			set:      []uint{1, 65, 130, 190, 250, 300, 380, 420, 480, 511},
+			startIdx: 511,
+			wantIdx:  511,
+			wantOk:   true,
+		},
+		{
+			name:     "last-2",
+			set:      []uint{1, 65, 130, 190, 250, 300, 380, 420, 480, 511},
+			del:      []uint{511},
+			startIdx: 511,
+			wantIdx:  0,
+			wantOk:   false,
+		},
+	}
+
+	for _, tc := range testCases {
+		var b BitSet
+		for _, u := range tc.set {
+			b.Set(u)
+		}
+
+		for _, u := range tc.del {
+			b.Clear(u) // without compact
+		}
+
+		idx, ok := b.NextSet(tc.startIdx)
+
+		if ok != tc.wantOk {
+			t.Errorf("NextSet, %s: got ok: %v, want: %v", tc.name, ok, tc.wantOk)
+		}
+		if idx != tc.wantIdx {
+			t.Errorf("NextSet, %s: got next idx: %d, want: %d", tc.name, idx, tc.wantIdx)
+		}
+	}
+}
+
+func TestNextSetMany(t *testing.T) {
+	testCases := []struct {
+		name string
+		//
+		set []uint
+		del []uint
+		//
+		buf      []uint
+		wantData []uint
+		//
+		startIdx uint
+		wantIdx  uint
+	}{
+		{
+			name:     "null",
+			set:      []uint{},
+			del:      []uint{},
+			buf:      make([]uint, 0, 512),
+			wantData: []uint{},
+			startIdx: 0,
+			wantIdx:  0,
+		},
+		{
+			name:     "zero",
+			set:      []uint{0},
+			del:      []uint{},
+			buf:      make([]uint, 0, 512),
+			wantData: []uint{0}, // bit #0 is set
+			startIdx: 0,
+			wantIdx:  0,
+		},
+		{
+			name:     "1,5",
+			set:      []uint{1, 5},
+			del:      []uint{},
+			buf:      make([]uint, 0, 512),
+			wantData: []uint{1, 5},
+			startIdx: 0,
+			wantIdx:  5,
+		},
+		{
+			name:     "many",
+			set:      []uint{1, 65, 130, 190, 250, 300, 380, 420, 480, 511},
+			del:      []uint{},
+			buf:      make([]uint, 0, 512),
+			wantData: []uint{1, 65, 130, 190, 250, 300, 380, 420, 480, 511},
+			startIdx: 0,
+			wantIdx:  511,
+		},
+		{
+			name:     "start idx",
+			set:      []uint{1, 65, 130, 190, 250, 300, 380, 420, 480, 511},
+			del:      []uint{},
+			buf:      make([]uint, 0, 512),
+			wantData: []uint{250, 300, 380, 420, 480, 511},
+			startIdx: 195,
+			wantIdx:  511,
+		},
+		{
+			name:     "zero buffer",
+			set:      []uint{1, 2, 3, 4, 511},
+			del:      []uint{},
+			buf:      make([]uint, 0), // buffer
+			wantData: []uint{},
+			startIdx: 0,
+			wantIdx:  0,
+		},
+		{
+			name:     "buffer too short, first word",
+			set:      []uint{1, 2, 3, 4, 5, 6, 7, 8, 9},
+			del:      []uint{},
+			buf:      make([]uint, 0, 5), // buffer
+			wantData: []uint{1, 2, 3, 4, 5},
+			startIdx: 0,
+			wantIdx:  5,
+		},
+		{
+			name:     "buffer too short",
+			set:      []uint{65, 66, 67, 68, 69, 70},
+			del:      []uint{},
+			buf:      make([]uint, 0, 5), // buffer
+			wantData: []uint{65, 66, 67, 68, 69},
+			startIdx: 0,
+			wantIdx:  69,
+		},
+		{
+			name:     "special, last return",
+			set:      []uint{1},
+			del:      []uint{1},          // delete without compact
+			buf:      make([]uint, 0, 5), // buffer
+			wantData: []uint{},
+			startIdx: 0,
+			wantIdx:  0,
+		},
+	}
+
+	for _, tc := range testCases {
+		var b BitSet
+		for _, u := range tc.set {
+			b.Set(u)
+		}
+
+		for _, u := range tc.del {
+			b.Clear(u) // without compact
+		}
+
+		idx, buf := b.NextSetMany(tc.startIdx, tc.buf)
+
+		if idx != tc.wantIdx {
+			t.Errorf("NextSetMany, %s: got next idx: %d, want: %d", tc.name, idx, tc.wantIdx)
+		}
+
+		if !reflect.DeepEqual(buf, tc.wantData) {
+			t.Errorf("NextSetMany, %s: returned buf is not equal as expected:\ngot:  %v\nwant: %v",
+				tc.name, buf, tc.wantData)
+		}
+	}
+}
+
+func TestAppendTo(t *testing.T) {
+	testCases := []struct {
+		name string
+		set  []uint
+		buf  []uint
+	}{
+		{
+			name: "null",
+			set:  nil,
+			buf:  nil,
+		},
+		{
+			name: "one",
+			set:  []uint{42},
+			buf:  make([]uint, 0, 5),
+		},
+		{
+			name: "many",
+			set:  []uint{1, 42, 55, 258, 7211, 54666},
+			buf:  make([]uint, 0),
+		},
+	}
+
+	for _, tc := range testCases {
+		var b BitSet
+		for _, u := range tc.set {
+			b.Set(u)
+		}
+
+		tc.buf = b.AppendTo(tc.buf)
+
+		if !reflect.DeepEqual(tc.buf, tc.set) {
+			t.Errorf("AppendTo, %s: returned buf is not equal as expected:\ngot:  %v\nwant: %v",
+				tc.name, tc.buf, tc.set)
+		}
+	}
+}
+
+func TestAsSlice(t *testing.T) {
+	testCases := []struct {
+		name string
+		set  []uint
+		buf  []uint
+	}{
+		{
+			name: "null",
+			set:  nil,
+			buf:  nil,
+		},
+		{
+			name: "one",
+			set:  []uint{42},
+			buf:  make([]uint, 1),
+		},
+		{
+			name: "many",
+			set:  []uint{1, 42, 55, 258, 7211, 54666},
+			buf:  make([]uint, 6),
+		},
+	}
+
+	for _, tc := range testCases {
+		var b BitSet
+		for _, u := range tc.set {
+			b.Set(u)
+		}
+
+		tc.buf = b.AsSlice(tc.buf)
+
+		if !reflect.DeepEqual(tc.buf, tc.set) {
+			t.Errorf("AsSlice, %s: returned buf is not equal as expected:\ngot:  %v\nwant: %v",
+				tc.name, tc.buf, tc.set)
+		}
+	}
+}
+
+func TestPanicAppendTo(t *testing.T) {
+	defer func() {
+		if r := recover(); r != nil {
+			t.Error("AppendTo with empty buf should not have caused a panic")
+		}
+	}()
+	v := New(1000)
+	v.Set(1000)
+	_ = v.AppendTo(nil)
+}
+
+func TestPanicAsSlice(t *testing.T) {
+	defer func() {
+		if r := recover(); r == nil {
+			t.Error("AsSlice with buf too small should have caused a panic")
+		}
+	}()
+	v := New(1000)
+	v.Set(1000)
+	_ = v.AsSlice(nil)
 }
 
 func TestSetTo(t *testing.T) {
@@ -420,9 +722,57 @@ func TestNullCount(t *testing.T) {
 	}
 }
 
+func TestMustNew(t *testing.T) {
+	testCases := []struct {
+		length uint
+		nwords int
+	}{
+		{
+			length: 0,
+			nwords: 0,
+		},
+		{
+			length: 1,
+			nwords: 1,
+		},
+		{
+			length: 64,
+			nwords: 1,
+		},
+		{
+			length: 65,
+			nwords: 2,
+		},
+		{
+			length: 512,
+			nwords: 8,
+		},
+		{
+			length: 513,
+			nwords: 9,
+		},
+	}
+
+	for _, tc := range testCases {
+		b := MustNew(tc.length)
+		if len(b.set) != tc.nwords {
+			t.Errorf("length = %d, len(b.set) got: %d, want: %d", tc.length, len(b.set), tc.nwords)
+		}
+	}
+}
+
+func TestPanicMustNew(t *testing.T) {
+	defer func() {
+		if r := recover(); r == nil {
+			t.Error("length too big should have caused a panic")
+		}
+	}()
+	MustNew(Cap())
+}
+
 func TestPanicDifferenceBNil(t *testing.T) {
 	var b *BitSet
-	var compare = New(10)
+	compare := New(10)
 	defer func() {
 		if r := recover(); r == nil {
 			t.Error("Nil First should should have caused a panic")
@@ -433,7 +783,7 @@ func TestPanicDifferenceBNil(t *testing.T) {
 
 func TestPanicDifferenceCompareNil(t *testing.T) {
 	var compare *BitSet
-	var b = New(10)
+	b := New(10)
 	defer func() {
 		if r := recover(); r == nil {
 			t.Error("Nil Second should should have caused a panic")
@@ -444,7 +794,7 @@ func TestPanicDifferenceCompareNil(t *testing.T) {
 
 func TestPanicUnionBNil(t *testing.T) {
 	var b *BitSet
-	var compare = New(10)
+	compare := New(10)
 	defer func() {
 		if r := recover(); r == nil {
 			t.Error("Nil First should should have caused a panic")
@@ -455,7 +805,7 @@ func TestPanicUnionBNil(t *testing.T) {
 
 func TestPanicUnionCompareNil(t *testing.T) {
 	var compare *BitSet
-	var b = New(10)
+	b := New(10)
 	defer func() {
 		if r := recover(); r == nil {
 			t.Error("Nil Second should should have caused a panic")
@@ -466,7 +816,7 @@ func TestPanicUnionCompareNil(t *testing.T) {
 
 func TestPanicIntersectionBNil(t *testing.T) {
 	var b *BitSet
-	var compare = New(10)
+	compare := New(10)
 	defer func() {
 		if r := recover(); r == nil {
 			t.Error("Nil First should should have caused a panic")
@@ -477,7 +827,7 @@ func TestPanicIntersectionBNil(t *testing.T) {
 
 func TestPanicIntersectionCompareNil(t *testing.T) {
 	var compare *BitSet
-	var b = New(10)
+	b := New(10)
 	defer func() {
 		if r := recover(); r == nil {
 			t.Error("Nil Second should should have caused a panic")
@@ -488,7 +838,7 @@ func TestPanicIntersectionCompareNil(t *testing.T) {
 
 func TestPanicSymmetricDifferenceBNil(t *testing.T) {
 	var b *BitSet
-	var compare = New(10)
+	compare := New(10)
 	defer func() {
 		if r := recover(); r == nil {
 			t.Error("Nil First should should have caused a panic")
@@ -499,7 +849,7 @@ func TestPanicSymmetricDifferenceBNil(t *testing.T) {
 
 func TestPanicSymmetricDifferenceCompareNil(t *testing.T) {
 	var compare *BitSet
-	var b = New(10)
+	b := New(10)
 	defer func() {
 		if r := recover(); r == nil {
 			t.Error("Nil Second should should have caused a panic")
@@ -752,7 +1102,6 @@ func TestInsertAtWithSet(t *testing.T) {
 		t.Error("66 should be set")
 		return
 	}
-
 }
 
 func TestInsertAt(t *testing.T) {
@@ -953,6 +1302,14 @@ func TestUnion(t *testing.T) {
 	}
 }
 
+func TestEmptyUnionCardinality(t *testing.T) {
+	a := New(0)
+	b := New(0)
+	if a.UnionCardinality(b) != 0 {
+		t.Error("UnionCardinality should be zero")
+	}
+}
+
 func TestInPlaceUnion(t *testing.T) {
 	a := New(100)
 	b := New(200)
@@ -1004,6 +1361,14 @@ func TestIntersection(t *testing.T) {
 	}
 }
 
+func TestEmptyIntersectionCardinality(t *testing.T) {
+	a := New(0)
+	b := New(0)
+	if a.IntersectionCardinality(b) != 0 {
+		t.Error("IntersectionCardinality should be zero")
+	}
+}
+
 func TestInplaceIntersection(t *testing.T) {
 	a := New(100)
 	b := New(200)
@@ -1059,6 +1424,14 @@ func TestDifference(t *testing.T) {
 	}
 }
 
+func TestEmptyDifferenceCardinality(t *testing.T) {
+	a := New(0)
+	b := New(0)
+	if a.DifferenceCardinality(b) != 0 {
+		t.Error("DifferenceCardinality should be zero")
+	}
+}
+
 func TestInPlaceDifference(t *testing.T) {
 	a := New(100)
 	b := New(200)
@@ -1114,6 +1487,14 @@ func TestSymmetricDifference(t *testing.T) {
 	}
 }
 
+func TestEmptySymmetricDifferenceCardinality(t *testing.T) {
+	a := New(0)
+	b := New(0)
+	if a.SymmetricDifferenceCardinality(b) != 0 {
+		t.Error("SymmetricDifferenceCardinality should be zero")
+	}
+}
+
 func TestInPlaceSymmetricDifference(t *testing.T) {
 	a := New(100)
 	b := New(200)
@@ -1279,13 +1660,13 @@ func TestMarshalUnmarshalBinaryByLittleEndian(t *testing.T) {
 func copyBinary(t *testing.T, from encoding.BinaryMarshaler, to encoding.BinaryUnmarshaler) {
 	data, err := from.MarshalBinary()
 	if err != nil {
-		t.Errorf(err.Error())
+		t.Error(err.Error())
 		return
 	}
 
 	err = to.UnmarshalBinary(data)
 	if err != nil {
-		t.Errorf(err.Error())
+		t.Error(err.Error())
 		return
 	}
 }
@@ -1296,14 +1677,14 @@ func TestMarshalUnmarshalJSON(t *testing.T) {
 		a.Set(10).Set(1001)
 		data, err := json.Marshal(a)
 		if err != nil {
-			t.Errorf(err.Error())
+			t.Error(err.Error())
 			return
 		}
 
 		b := new(BitSet)
 		err = json.Unmarshal(data, b)
 		if err != nil {
-			t.Errorf(err.Error())
+			t.Error(err.Error())
 			return
 		}
 
@@ -1317,14 +1698,14 @@ func TestMarshalUnmarshalJSON(t *testing.T) {
 		a := New(1010).Set(10).Set(1001)
 		data, err := json.Marshal(a)
 		if err != nil {
-			t.Errorf(err.Error())
+			t.Error(err.Error())
 			return
 		}
 
 		b := new(BitSet)
 		err = json.Unmarshal(data, b)
 		if err != nil {
-			t.Errorf(err.Error())
+			t.Error(err.Error())
 			return
 		}
 
@@ -1340,7 +1721,7 @@ func TestMarshalUnmarshalJSONWithTrailingData(t *testing.T) {
 	a := New(1010).Set(10).Set(1001)
 	data, err := json.Marshal(a)
 	if err != nil {
-		t.Errorf(err.Error())
+		t.Error(err.Error())
 		return
 	}
 
@@ -1351,7 +1732,7 @@ func TestMarshalUnmarshalJSONWithTrailingData(t *testing.T) {
 	b := new(BitSet)
 	err = json.Unmarshal(data, b)
 	if err != nil {
-		t.Errorf(err.Error())
+		t.Error(err.Error())
 		return
 	}
 
@@ -1368,14 +1749,14 @@ func TestMarshalUnmarshalJSONByStdEncoding(t *testing.T) {
 	a := New(1010).Set(10).Set(1001)
 	data, err := json.Marshal(a)
 	if err != nil {
-		t.Errorf(err.Error())
+		t.Error(err.Error())
 		return
 	}
 
 	b := new(BitSet)
 	err = json.Unmarshal(data, b)
 	if err != nil {
-		t.Errorf(err.Error())
+		t.Error(err.Error())
 		return
 	}
 
@@ -1434,6 +1815,22 @@ func TestFrom(t *testing.T) {
 	}
 }
 
+func TestWords(t *testing.T) {
+	b := new(BitSet)
+	c := b.Words()
+	outType := fmt.Sprintf("%T", c)
+	expType := "[]uint64"
+	if outType != expType {
+		t.Error("Expecting type: ", expType, ", gotf:", outType)
+		return
+	}
+	if len(c) != 0 {
+		t.Error("The slice should be empty")
+		return
+	}
+}
+
+// Bytes is deprecated
 func TestBytes(t *testing.T) {
 	b := new(BitSet)
 	c := b.Bytes()
@@ -1534,6 +1931,7 @@ func TestRankSelect(t *testing.T) {
 		return
 	}
 }
+
 func TestFlip(t *testing.T) {
 	b := new(BitSet)
 	c := b.Flip(11)
@@ -1782,7 +2180,6 @@ func TestDeleteWithBitSetInstance(t *testing.T) {
 				t.Errorf("Expected index %d to not be set, but was", i)
 			}
 		}
-
 	}
 }
 
@@ -1963,3 +2360,748 @@ func TestSetAll(t *testing.T) {
 		test(fmt.Sprintf("length %d", length), New(length), length)
 	}
 }
+
+func TestShiftLeft(t *testing.T) {
+	data := []uint{5, 28, 45, 72, 89, 560}
+
+	test := func(name string, bits uint) {
+		t.Run(name, func(t *testing.T) {
+			b := New(200)
+			for _, i := range data {
+				b.Set(i)
+			}
+
+			b.ShiftLeft(bits)
+
+			if int(b.Count()) != len(data) {
+				t.Error("bad bits count")
+			}
+
+			for _, i := range data {
+				if !b.Test(i + bits) {
+					t.Errorf("bit %v is not set", i+bits)
+				}
+			}
+		})
+	}
+
+	test("zero", 0)
+	test("no page change", 19)
+	test("shift to full page", 38)
+	test("full page shift", 64)
+	test("no page split", 80)
+	test("with page split", 114)
+	test("with extension", 242)
+	test("with extra word", 16)
+}
+
+func TestShiftRight(t *testing.T) {
+	data := []uint{5, 28, 45, 72, 89}
+
+	test := func(name string, bits uint) {
+		t.Run(name, func(t *testing.T) {
+			b := New(200)
+			for _, i := range data {
+				b.Set(i)
+			}
+
+			b.ShiftRight(bits)
+
+			count := 0
+			for _, i := range data {
+				if i >= bits {
+					count++
+
+					if !b.Test(i - bits) {
+						t.Errorf("bit %v is not set", i-bits)
+					}
+				}
+			}
+
+			if int(b.Count()) != count {
+				t.Errorf("bad bits count: expected %d, got %d", count, b.Count())
+			}
+		})
+	}
+
+	test("zero", 0)
+	test("no page change", 3)
+	test("no page split", 20)
+	test("with page split", 40)
+	test("full page shift", 64)
+	test("with extension", 70)
+	test("full shift", 89)
+	test("remove all", 242)
+}
+
+func TestShiftRightFull(t *testing.T) {
+	testCases := []struct{
+		data []uint
+		shiftDistance uint
+	}{
+		{
+			[]uint{20}, 20,
+		},
+		{
+			[]uint{0, 20, 40, 1260, 1280}, 1,
+		},
+		{
+			[]uint{0, 20, 40, 1260, 1280}, 1281,
+		},
+	}
+
+	test := func(data []uint, shiftDistance uint) {
+		b := New(0)
+		for i := range data {
+			b.Set(data[i])
+		}
+		b.ShiftRight(shiftDistance)
+		for i := range data {
+			shiftedBit := int(data[i])-int(shiftDistance)
+			if shiftedBit >= 0 {
+				if !b.Test(uint(shiftedBit)) {
+					t.Errorf("bit %d should be set after ShiftRight(%d) if bit %d was set prior", data[i]-shiftDistance, shiftDistance, data[i])
+				}
+			}
+		}
+	}
+
+	for i := range testCases {
+		test(testCases[i].data, testCases[i].shiftDistance)
+	}
+}
+
+func TestWord(t *testing.T) {
+	data := []uint64{0x0bfd85fc01af96dd, 0x3fe212a7eae11414, 0x7aa412221245dee1, 0x557092c1711306d5}
+	testCases := map[string]struct {
+		index    uint
+		expected uint64
+	}{
+		"first word": {
+			index:    0,
+			expected: 0x0bfd85fc01af96dd,
+		},
+		"third word": {
+			index:    128,
+			expected: 0x7aa412221245dee1,
+		},
+		"off the edge": {
+			index:    256,
+			expected: 0,
+		},
+		"way off the edge": {
+			index:    6346235235,
+			expected: 0,
+		},
+		"split between two words": {
+			index:    96,
+			expected: 0x1245dee13fe212a7,
+		},
+		"partly off edge": {
+			index:    254,
+			expected: 1,
+		},
+		"skip one nibble": {
+			index:    4,
+			expected: 0x40bfd85fc01af96d,
+		},
+		"slightly offset results": {
+			index:    65,
+			expected: 0x9ff10953f5708a0a,
+		},
+	}
+
+	for name, testCase := range testCases {
+		t.Run(name, func(t *testing.T) {
+			bitSet := From(data)
+			output := bitSet.GetWord64AtBit(testCase.index)
+
+			if output != testCase.expected {
+				t.Errorf("Word should have returned %d for input %d, but returned %d", testCase.expected, testCase.index, output)
+			}
+		})
+	}
+}
+
+func TestPreviousSet(t *testing.T) {
+	v := New(128)
+	v.Set(0)
+	v.Set(2)
+	v.Set(4)
+	v.Set(120)
+	for _, tt := range []struct {
+		index     uint
+		want      uint
+		wantFound bool
+	}{
+		{0, 0, true},
+		{1, 0, true},
+		{2, 2, true},
+		{3, 2, true},
+		{4, 4, true},
+		{5, 4, true},
+		{100, 4, true},
+		{120, 120, true},
+		{121, 120, true},
+		{1024, 0, false},
+	} {
+		t.Run(fmt.Sprintf("@%d", tt.index), func(t *testing.T) {
+			got, found := v.PreviousSet(tt.index)
+			if got != tt.want || found != tt.wantFound {
+				t.Errorf("PreviousSet(%d) = %d, %v, want %d, %v", tt.index, got, found, tt.want, tt.wantFound)
+			}
+		})
+	}
+	v.ClearAll()
+	for _, tt := range []struct {
+		index     uint
+		want      uint
+		wantFound bool
+	}{
+		{0, 0, false},
+		{120, 0, false},
+		{1024, 0, false},
+	} {
+		t.Run(fmt.Sprintf("@%d", tt.index), func(t *testing.T) {
+			got, found := v.PreviousSet(tt.index)
+			if got != tt.want || found != tt.wantFound {
+				t.Errorf("PreviousSet(%d) = %d, %v, want %d, %v", tt.index, got, found, tt.want, tt.wantFound)
+			}
+		})
+	}
+}
+
+func TestPreviousClear(t *testing.T) {
+	v := New(128)
+	v.Set(0)
+	v.Set(2)
+	v.Set(4)
+	v.Set(120)
+	for _, tt := range []struct {
+		index     uint
+		want      uint
+		wantFound bool
+	}{
+		{0, 0, false},
+		{1, 1, true},
+		{2, 1, true},
+		{3, 3, true},
+		{4, 3, true},
+		{5, 5, true},
+		{100, 100, true},
+		{120, 119, true},
+		{121, 121, true},
+		{1024, 0, false},
+	} {
+		t.Run(fmt.Sprintf("@%d", tt.index), func(t *testing.T) {
+			got, found := v.PreviousClear(tt.index)
+			if got != tt.want || found != tt.wantFound {
+				t.Errorf("PreviousClear(%d) = %d, %v, want %d, %v", tt.index, got, found, tt.want, tt.wantFound)
+			}
+		})
+	}
+	v.SetAll()
+	for _, tt := range []struct {
+		index     uint
+		want      uint
+		wantFound bool
+	}{
+		{0, 0, false},
+		{120, 0, false},
+		{1024, 0, false},
+	} {
+		t.Run(fmt.Sprintf("@%d", tt.index), func(t *testing.T) {
+			got, found := v.PreviousClear(tt.index)
+			if got != tt.want || found != tt.wantFound {
+				t.Errorf("PreviousClear(%d) = %d, %v, want %d, %v", tt.index, got, found, tt.want, tt.wantFound)
+			}
+		})
+	}
+}
+
+func TestBitSetOnesBetween(t *testing.T) {
+	testCases := []struct {
+		name     string
+		input    *BitSet
+		from     uint
+		to       uint
+		expected uint
+	}{
+		{"empty range", New(64).Set(0).Set(1), 5, 5, 0},
+		{"invalid range", New(64).Set(0).Set(1), 5, 3, 0},
+		{"single word", New(64).Set(1).Set(2).Set(3), 1, 3, 2},
+		{"single word full", New(64).Set(0).Set(1).Set(2).Set(3), 0, 4, 4},
+		{"cross word boundary", New(128).Set(63).Set(64).Set(65), 63, 66, 3},
+		{"multiple words", New(256).Set(0).Set(63).Set(64).Set(127).Set(128), 0, 129, 5},
+		{"large gap", New(256).Set(0).Set(100).Set(200), 0, 201, 3},
+	}
+
+	for _, tc := range testCases {
+		t.Run(tc.name, func(t *testing.T) {
+			got := tc.input.OnesBetween(tc.from, tc.to)
+			if got != tc.expected {
+				t.Errorf("OnesBetween(%d, %d) = %d, want %d",
+					tc.from, tc.to, got, tc.expected)
+			}
+		})
+	}
+
+	// Property-based testing
+	const numTests = 1e5
+	seed := time.Now().UnixNano()
+	rng := rand.New(rand.NewSource(seed))
+	t.Logf("Seed: %d", seed)
+
+	for i := 0; i < numTests; i++ {
+		size := uint(rng.Intn(1024) + 64)
+		bs := New(size)
+
+		// Set random bits
+		for j := 0; j < int(size/4); j++ {
+			bs.Set(uint(rng.Intn(int(size))))
+		}
+
+		// Generate random range
+		from := uint(rng.Intn(int(size)))
+		to := from + uint(rng.Intn(int(size-from)))
+
+		// Compare with naive implementation
+		got := bs.OnesBetween(from, to)
+		want := uint(0)
+		for j := from; j < to; j++ {
+			if bs.Test(j) {
+				want++
+			}
+		}
+
+		if got != want {
+			t.Errorf("Case %d: OnesBetween(%d, %d) = %d, want %d",
+				i, from, to, got, want)
+		}
+	}
+}
+
+func BenchmarkBitSetOnesBetween(b *testing.B) {
+	sizes := []int{64, 256, 1024, 4096, 16384}
+	densities := []float64{0.1, 0.5, 0.9} // Different bit densities to test
+	rng := rand.New(rand.NewSource(42))
+
+	for _, size := range sizes {
+		for _, density := range densities {
+			// Create bitset with given density
+			bs := New(uint(size))
+			for i := 0; i < int(float64(size)*density); i++ {
+				bs.Set(uint(rng.Intn(size)))
+			}
+
+			// Generate random ranges
+			ranges := make([][2]uint, 1000)
+			for i := range ranges {
+				from := uint(rng.Intn(size))
+				to := from + uint(rng.Intn(size-int(from)))
+				ranges[i] = [2]uint{from, to}
+			}
+
+			name := fmt.Sprintf("size=%d/density=%.1f", size, density)
+			b.Run(name, func(b *testing.B) {
+				for i := 0; i < b.N; i++ {
+					r := ranges[i%len(ranges)]
+					_ = bs.OnesBetween(r[0], r[1])
+				}
+			})
+		}
+	}
+}
+
+func generatePextTestCases(n int) [][2]uint64 {
+	cases := make([][2]uint64, n)
+	for i := range cases {
+		cases[i][0] = rand.Uint64()
+		cases[i][1] = rand.Uint64()
+	}
+	return cases
+}
+
+func BenchmarkPEXT(b *testing.B) {
+	// Generate test cases
+	testCases := generatePextTestCases(1000)
+
+	b.ResetTimer()
+
+	var r uint64
+	for i := 0; i < b.N; i++ {
+		tc := testCases[i%len(testCases)]
+		r = pext(tc[0], tc[1])
+	}
+	_ = r // prevent optimization
+}
+
+func BenchmarkPDEP(b *testing.B) {
+	// Generate test cases
+	testCases := generatePextTestCases(1000)
+
+	b.ResetTimer()
+
+	var r uint64
+	for i := 0; i < b.N; i++ {
+		tc := testCases[i%len(testCases)]
+		r = pdep(tc[0], tc[1])
+	}
+	_ = r // prevent optimization
+}
+
+func TestPext(t *testing.T) {
+	const numTests = 1e6
+	seed := time.Now().UnixNano()
+	rng := rand.New(rand.NewSource(seed))
+	t.Logf("Seed: %d", seed)
+
+	for i := 0; i < numTests; i++ {
+		w := rng.Uint64()
+		m := rng.Uint64()
+		result := pext(w, m)
+		popCount := bits.OnesCount64(m)
+
+		// Test invariants
+		if popCount > 0 && result >= (uint64(1)<<popCount) {
+			t.Fatalf("Case %d: result %x exceeds maximum possible value for mask with popcount %d",
+				i, result, popCount)
+		}
+
+		if bits.OnesCount64(result) > bits.OnesCount64(w&m) {
+			t.Fatalf("Case %d: result has more 1s than masked input: result=%x, input&mask=%x",
+				i, result, w&m)
+		}
+
+		// Test that extracted bits preserve relative ordering:
+		// For each bit position that's set in the mask (m):
+		// 1. Extract a bit from result (resultCopy&1)
+		// 2. Get corresponding input bit from w (w>>j&1)
+		// 3. XOR them - if different, bits weren't preserved correctly
+		resultCopy := result
+		for j := 0; j < 64; j++ {
+			// Check if mask bit is set at position j
+			if m&(uint64(1)<<j) != 0 {
+				// resultCopy&1 gets LSB of remaining result bits
+				// w>>j&1 gets bit j from original input
+				// XOR (^) checks if they match
+				if (resultCopy&1)^(w>>j&1) != 0 {
+					t.Fatalf("Case %d: bit ordering violation at position %d", i, j)
+				}
+				// Shift to examine next bit in packed result
+				resultCopy >>= 1
+			}
+		}
+	}
+}
+
+func TestPdep(t *testing.T) {
+	const numTests = 1e6
+	seed := time.Now().UnixNano()
+	rng := rand.New(rand.NewSource(seed))
+	t.Logf("Seed: %d", seed)
+
+	for i := 0; i < numTests; i++ {
+		w := rng.Uint64() // value to deposit
+		m := rng.Uint64() // mask
+		result := pdep(w, m)
+		popCount := bits.OnesCount64(m)
+
+		// Test invariants
+		if result&^m != 0 {
+			t.Fatalf("Case %d: result %x has bits set outside of mask %x",
+				i, result, m)
+		}
+
+		if bits.OnesCount64(result) > bits.OnesCount64(w) {
+			t.Fatalf("Case %d: result has more 1s than input: result=%x, input=%x",
+				i, result, w)
+		}
+
+		// Verify by using PEXT to extract bits back
+		// The composition of PEXT(PDEP(x,m),m) should equal x masked to popcount bits
+		extracted := pext(result, m)
+		maskBits := (uint64(1) << popCount) - 1
+		if (extracted & maskBits) != (w & maskBits) {
+			t.Fatalf("Case %d: PEXT(PDEP(w,m),m) != w: got=%x, want=%x (w=%x, m=%x)",
+				i, extracted&maskBits, w&maskBits, w, m)
+		}
+	}
+}
+
+func TestBitSetExtract(t *testing.T) {
+	// Property-based tests
+	const numTests = 1e4
+	seed := time.Now().UnixNano()
+	rng := rand.New(rand.NewSource(seed))
+	t.Logf("Seed: %d", seed)
+
+	for i := 0; i < numTests; i++ {
+		// Create random bitsets
+		size := uint(rng.Intn(1024) + 64) // Random size between 64-1087 bits
+		src := New(size)
+		mask := New(size)
+		dst := New(size)
+
+		// Set random bits
+		for j := 0; j < int(size/4); j++ {
+			src.Set(uint(rng.Intn(int(size))))
+			mask.Set(uint(rng.Intn(int(size))))
+		}
+
+		// Extract bits
+		src.ExtractTo(mask, dst)
+
+		// Test invariants
+		if dst.Count() > src.IntersectionCardinality(mask) {
+			t.Errorf("Case %d: result has more 1s than masked input", i)
+		}
+
+		// Test bits are properly extracted and packed
+		pos := uint(0)
+		for j := uint(0); j < size; j++ {
+			if mask.Test(j) {
+				if src.Test(j) != dst.Test(pos) {
+					t.Errorf("Case %d: bit ordering violation at source position %d", i, j)
+				}
+				pos++
+			}
+		}
+	}
+
+	// Keep existing test cases
+	testCases := []struct {
+		name     string
+		src      *BitSet // source bits
+		mask     *BitSet // mask bits
+		expected *BitSet // expected extracted bits
+	}{
+		{
+			name:     "single bit",
+			src:      New(8).Set(1), // 0b01
+			mask:     New(8).Set(1), // 0b01
+			expected: New(8).Set(0), // 0b1
+		},
+		{
+			name:     "two sequential bits",
+			src:      New(8).Set(0).Set(1), // 0b11
+			mask:     New(8).Set(0).Set(1), // 0b11
+			expected: New(8).Set(0).Set(1), // 0b11
+		},
+		{
+			name:     "sparse bits",
+			src:      New(16).Set(0).Set(10),        // 0b10000000001
+			mask:     New(16).Set(0).Set(5).Set(10), // 0b10000100001
+			expected: New(8).Set(0).Set(2),          // 0b101
+		},
+		{
+			name:     "masked off bits",
+			src:      New(8).Set(0).Set(1).Set(2).Set(3), // 0b1111
+			mask:     New(8).Set(0).Set(2),               // 0b0101
+			expected: New(8).Set(0).Set(1),               // 0b11
+		},
+		{
+			name:     "cross word boundary",
+			src:      New(128).Set(63).Set(64).Set(65),
+			mask:     New(128).Set(63).Set(64).Set(65),
+			expected: New(8).Set(0).Set(1).Set(2),
+		},
+		{
+			name:     "large gap",
+			src:      New(256).Set(0).Set(100).Set(200),
+			mask:     New(256).Set(0).Set(100).Set(200),
+			expected: New(8).Set(0).Set(1).Set(2),
+		},
+		{
+			name:     "extracting zeros",
+			src:      New(8),                      // 0b00
+			mask:     New(8).Set(0).Set(1).Set(2), // 0b111
+			expected: New(8),                      // 0b000
+		},
+	}
+
+	for _, tc := range testCases {
+		t.Run(tc.name, func(t *testing.T) {
+			dst := New(tc.expected.Len())
+			tc.src.ExtractTo(tc.mask, dst)
+			if !dst.Equal(tc.expected) {
+				t.Errorf("got %v, expected %v", dst, tc.expected)
+			}
+
+			// Verify inverse relationship within the mask bits
+			deposited := New(tc.src.Len())
+			dst.DepositTo(tc.mask, deposited)
+
+			// Only bits selected by the mask should match between source and deposited
+			maskedSource := tc.src.Intersection(tc.mask)
+			maskedDeposited := deposited.Intersection(tc.mask)
+
+			if !maskedSource.Equal(maskedDeposited) {
+				t.Error("DepositTo(ExtractTo(x,m),m) doesn't preserve masked source bits")
+			}
+		})
+	}
+}
+
+func TestBitSetDeposit(t *testing.T) {
+	// Property-based tests
+	const numTests = 1e4
+	seed := time.Now().UnixNano()
+	rng := rand.New(rand.NewSource(seed))
+	t.Logf("Seed: %d", seed)
+
+	for i := 0; i < numTests; i++ {
+		// Create random bitsets
+		size := uint(rng.Intn(1024) + 64) // Random size between 64-1087 bits
+		src := New(size)
+		mask := New(size)
+		dst := New(size)
+
+		// Set random bits
+		for j := 0; j < int(size/4); j++ {
+			src.Set(uint(rng.Intn(int(mask.Count() + 1))))
+			mask.Set(uint(rng.Intn(int(size))))
+		}
+
+		// Deposit bits
+		src.DepositTo(mask, dst)
+
+		// Test invariants
+		if dst.Count() > src.Count() {
+			t.Errorf("Case %d: result has more 1s than input", i)
+		}
+
+		if (dst.Bytes()[0] &^ mask.Bytes()[0]) != 0 {
+			t.Errorf("Case %d: result has bits set outside of mask", i)
+		}
+
+		// Extract bits back and verify
+		extracted := New(size)
+		dst.ExtractTo(mask, extracted)
+		maskBits := New(size)
+		for j := uint(0); j < mask.Count(); j++ {
+			maskBits.Set(j)
+		}
+		srcMasked := src.Clone()
+		srcMasked.InPlaceIntersection(maskBits)
+		if !extracted.Equal(srcMasked) {
+			t.Errorf("Case %d: ExtractTo(DepositTo(x,m),m) != x", i)
+		}
+	}
+
+	// Keep existing test cases
+	testCases := []struct {
+		name     string
+		src      *BitSet // source bits (packed in low positions)
+		mask     *BitSet // mask bits (positions to deposit into)
+		dst      *BitSet // destination bits (initially set)
+		expected *BitSet // expected result
+	}{
+		{
+			name:     "sparse bits",
+			src:      New(8).Set(0),        // 0b01
+			mask:     New(8).Set(0).Set(5), // 0b100001
+			expected: New(8).Set(0),        // 0b000001
+		},
+		{
+			name:     "masked off bits",
+			src:      New(8).Set(0).Set(1), // 0b11
+			mask:     New(8).Set(0).Set(2), // 0b101
+			expected: New(8).Set(0).Set(2), // 0b101
+		},
+		{
+			name:     "cross word boundary",
+			src:      New(8).Set(0).Set(1),     // 0b11
+			mask:     New(128).Set(63).Set(64), // bits across word boundary
+			expected: New(128).Set(63).Set(64), // bits deposited across boundary
+		},
+		{
+			name:     "large gaps",
+			src:      New(8).Set(0).Set(1),     // 0b11
+			mask:     New(128).Set(0).Set(100), // widely spaced bits
+			expected: New(128).Set(0).Set(100), // deposited into sparse positions
+		},
+		{
+			name:     "depositing zeros",
+			src:      New(8),                      // 0b00
+			mask:     New(8).Set(0).Set(1).Set(2), // 0b111
+			expected: New(8),                      // 0b000
+		},
+		{
+			name:     "preserve unmasked bits",
+			src:      New(8),                      // empty source
+			mask:     New(8),                      // empty mask
+			dst:      New(8).Set(1).Set(2).Set(3), // dst has some bits set
+			expected: New(8).Set(1).Set(2).Set(3), // should remain unchanged
+		},
+		{
+			name:     "preserve bits outside mask within word",
+			src:      New(8).Set(0),                      // source has bit 0 set
+			mask:     New(8).Set(1),                      // only depositing into bit 1
+			dst:      New(8).Set(0).Set(2).Set(3),        // dst has bits 0,2,3 set
+			expected: New(8).Set(0).Set(1).Set(2).Set(3), // bits 0,2,3 should remain, bit 1 should be set
+		},
+	}
+
+	for _, tc := range testCases {
+		t.Run(tc.name, func(t *testing.T) {
+			var dst *BitSet
+			if tc.dst == nil {
+				dst = New(tc.expected.Len())
+			} else {
+				dst = tc.dst.Clone()
+			}
+			tc.src.DepositTo(tc.mask, dst)
+			if !dst.Equal(tc.expected) {
+				t.Errorf("got %v, expected %v", dst, tc.expected)
+			}
+
+			// Verify inverse relationship for set bits up to mask cardinality
+			extracted := New(tc.src.Len())
+			dst.ExtractTo(tc.mask, extracted)
+
+			if !extracted.Equal(tc.src) {
+				t.Error("ExtractTo(DepositTo(x,m),m) doesn't preserve source bits that were selected by mask")
+			}
+		})
+	}
+}
+
+func BenchmarkBitSetExtractDeposit(b *testing.B) {
+	sizes := []int{64, 256, 1024, 4096, 16384, 2 << 15}
+	rng := rand.New(rand.NewSource(42)) // fixed seed for reproducibility
+
+	for _, size := range sizes {
+		// Create source with random bits
+		src := New(uint(size))
+		for i := 0; i < size/4; i++ { // Set ~25% of bits
+			src.Set(uint(rng.Intn(size)))
+		}
+
+		// Create mask with random bits
+		mask := New(uint(size))
+		for i := 0; i < size/4; i++ {
+			mask.Set(uint(rng.Intn(size)))
+		}
+
+		b.Run(fmt.Sprintf("size=%d/fn=ExtractTo", size), func(b *testing.B) {
+			dst := New(uint(size))
+			b.ReportAllocs()
+			b.ResetTimer()
+			for i := 0; i < b.N; i++ {
+				src.ExtractTo(mask, dst)
+				dst.ClearAll()
+			}
+		})
+
+		b.Run(fmt.Sprintf("size=%d/fn=DepositTo", size), func(b *testing.B) {
+			dst := New(uint(size))
+			b.ReportAllocs()
+			b.ResetTimer()
+			for i := 0; i < b.N; i++ {
+				src.DepositTo(mask, dst)
+				dst.ClearAll()
+			}
+		})
+	}
+}
